//
//  BrowserAppletFrame.java
//
//  Copyright (c) 1996, Microsoft Corporation
//

package com.ms.applet;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.image.ImageProducer;
import java.applet.*;
import java.net.*;
import com.ms.net.wininet.WininetStreamHandlerFactory;
import com.ms.applet.AppletAudioClip;
import com.ms.awt.image.ByteArrayImageSource;
import com.ms.net.wininet.URLUtils;
import sun.awt.image.URLImageSource;

class ClassLoaderEntry {
    int countRefs;
    URL codebase;
    AppletBaseClassLoader loader;

    ClassLoaderEntry(URL codebase, AppletBaseClassLoader loader)
    {
        this.countRefs = 1;
        this.codebase = codebase;
        this.loader = loader;
    }
}

public
class BrowserAppletFrame extends Frame
                   implements AppletStub, Runnable, IJavaObjectLoader
{
    /**
     * Parent window to show this applet in. AWT looks at this when creating
     * our child window.  Moving this field will require a rebuild of AWT!
     */
    int hwnd;

    Hashtable atts;

    /**
     * The document url.
     */
    URL documentURL;

    /**
     * The base url.
     */
    URL baseURL;

    /**
     * The applet (if loaded).
     */
    Applet applet;

    /**
     * The classloader for the applet.
     */
    AppletBaseClassLoader loader;

    /**
     * The current status. Either: APPLET_DISPOSE, APPLET_LOAD,
     * APPLET_INIT, APPLET_START.
     */
    int status;

    /**
     * IDispatch interface for applet.
     */
    int pdispApplet;

    /**
     * The thread for the applet.
     */
    Thread handler;

    /**
     * The applet size.
     */
    Dimension appletSize = new Dimension(0, 0);

    /* applet event ids */
    public final static int APPLET_DISPOSE = 0;
    public final static int APPLET_LOAD = 1;
    public final static int APPLET_INIT = 2;
    public final static int APPLET_START = 3;
    public final static int APPLET_STOP = 4;
    public final static int APPLET_DESTROY = 5;

    /* Windows messages */
    final static int WM_USER = 0x400;
    final static int HOSTFRAME_SHOWSTATUS = WM_USER+1;
    final static int HOSTFRAME_REPAINTSTATUS = WM_USER+2;
    final static int HOSTFRAME_SHOWDOCUMENT = WM_USER+3;
    final static int HOSTFRAME_SETDOWNLOADSTATE = WM_USER+4;

    /* resource string ids */
    final static int STATUS_INITED = 0x100;
    final static int STATUS_STARTED = 0x101;
    final static int STATUS_STOPPED = 0x102;
    final static int STATUS_DESTROYED = 0x103;
    final static int STATUS_OPENING_CLASS = 0x104;
    final static int STATUS_OPENING_CABINET = 0x105;

    /*
     * Hooks to manipulate the applet's IDispatch pointer.
     */

    public static native int getAppletDispatch(Applet applet);
    public static native void releaseAppletDispatch(int pDispatch);

     // from IJavaObjectLoader
    public int deliverJavaObjectDispatch()
    {
        if (pdispApplet != 0 && (status == APPLET_START)) {
            return pdispApplet;
        } else {
            return 0;
        }
    }

    /*
     * Puts the specified name/value pair into the applet's parameter bag.
     */
     // from IJavaObjectLoader
    public void loadParameter(String name, String value)
    {
        if (atts == null)
            atts = new Hashtable();

        // For Netscape compatiblity, we must remove leading/trailing spaces.
        value = value.trim();

        // For Netscape compatiblity, we must remove any control chars.
        int i;
        int j = 0;
        int len = value.length();
        char buf[] = new char[len];
        for (i = 0; i < len; i++) {
            char ch = value.charAt(i);
            if (ch >= 0x0020) {
                buf[j++] = ch;
            }
        }
        value = new String(buf, 0, j);

        atts.put(name.toLowerCase(), value);
    }

    /**
     * Massages the specified URL specification to conform to what the Java
     * libraries expect.
     */

    String massageURLSpec(String spec)
    {
        // IE gives us encoded URL strings.  (A space is %20.)
        // Canonicalize it with the Wininet API.  Remember to encode
        // the string again before passing it to Wininet or UrlMon.
        //
        // We'll leave http: URLs encoded.  It's up to the applet to deal with
        // encoding issues itself which is compatible with the default Sun
        // http: handler.
        if (!spec.startsWith("http:")) {
            String specNew = URLUtils.canonicalizeURL(spec,
                             URLUtils.ICU_DECODE | URLUtils.ICU_NO_ENCODE);
            if (specNew != null)
                spec = specNew;
        }

        // If the string after the file: starts with two or less forward
        // slashes, then conver this to three slashes.  The browser calls
        // InternetCanonicalizeURL which will always return file: URLs with
        // this format.  However, the Java URL parsing code ends up treating
        // the drive letter as a host which breaks a bunch of code including
        // the SecurityManager.
        if (spec.startsWith("file:")) {
            int i;
            int length = spec.length();
            for (i = 5; i < length; i++) {
                if (spec.charAt(i) != '/')
                    break;
            }
            if (i < 8) {
                spec = "file:///" + spec.substring(i);
            }
        }

        // For compatibility with existing Java applets, we'll
        // replace afll backslashes with forward slashes.  However, we
        // don't want to lose UNC information, so we need to special
        // case this.
        if (spec.startsWith("file:///\\\\")) {
            spec = "file:///\\\\" + spec.substring(10).replace('\\', '/');
        } else {
            spec = spec.replace('\\', '/');
        }

        return spec;
    }


    /**
     * Called by JavaObjectLoaderHost after all parameters have been loaded; the applet
     * can now be kicked off.
     */
     // from IJavaObjectLoader
    public void init(String document, int hwnd, IJavaObjectLoaderHost host)
    {
        String att;

        firstTimeInit();

        jolHost = host;

        this.hwnd = hwnd;

        try {
            com.ms.com.ComLib.declareMessagePumpThread();
        } catch (Throwable t) {
        }

        try {
            documentURL = new URL(massageURLSpec(document));
        } catch (MalformedURLException e) {
        }

        att = getParameter("codebase");

        if (att != null) {
            if (!att.endsWith("/")) {
                att += "/";
            }
            try {
                baseURL = new URL(massageURLSpec(att));
            } catch (MalformedURLException e) {
            }
        }

        if (baseURL == null) {
            baseURL = documentURL;
        }

        // Netscape's applets have a default background color of light gray.
        // A color related to the page would be preferable but some applets
        // don't paint properly without gray so we'll be compatible.
        setBackground(Color.lightGray);

        try {
            appletSize.width = Integer.valueOf(getParameter("width")).intValue();
        } catch (Exception e) {
            //  appletSize.width set to zero when allocated.
        }

        try {
            appletSize.height = Integer.valueOf(getParameter("height")).intValue();
        } catch (Exception e) {
            //  appletSize.height set to zero when allocated.
        }

        resize(appletSize);

        // Create a thread group for the applet, and start a new
        // thread to load the applet.
        String nm = "applet-" + getParameter("code");
        handler = new Thread(new AppletThreadGroup("group " + nm), this, "thread " + nm);
        handler.start();
    }

     // from IJavaObjectLoader2
    public void resizeApplet(int width, int height)
    {
        if (width != appletSize.width || height != appletSize.height) {
            appletSize.width = width;
            appletSize.height = height;
            resize(appletSize);
        }
    }

    /**
     * Event Queue
     */
    Event queue = null;

    /**
     * Send an event.
     */
     // from IJavaObjectLoader
    public void sendEvent(int id) {
        queueEvent(new Event(null, id, null));
    }

    /**
     * Send an event. Queue it for execution by the handler thread.
     */
    protected synchronized void queueEvent(Event evt) {
        if (queue == null) {
            evt.target = queue;
            queue = evt;
            notifyAll();
        } else {
            Event q = queue;
            for (; q.target != null ; q = (Event)q.target);
            q.target = evt;
        }
    }

    /**
     * Get an event from the queue.
     */
    synchronized Event getNextEvent() throws InterruptedException {
        while (queue == null) {
            wait();
        }
        Event evt = queue;
        queue = (Event)queue.target;
        evt.target = this;
        return evt;
    }

    static Vector frames = new Vector();

    public void run()
    {
        frames.addElement(this);

        try {

            // Bump up the priority, so that applet events are processed in
            // a timely manner.
            int priority = Thread.currentThread().getPriority();
            Thread.currentThread().setPriority(priority + 1);

            show();

            while (true) {
                Event evt;
                try {
                    evt = getNextEvent();
                } catch (InterruptedException e) {
                    // Bailing out...
                    return;
                }

                try {

                    switch (evt.id) {
                      case APPLET_LOAD:
                        if (status != APPLET_DISPOSE) {
                            break;
                        }

                        loader = getClassLoader();

                        // Create the applet using the class loader
                        String code = getParameter("code");
                        if (code.endsWith(".class")) {
                            code = code.substring(0, code.length() - 6).replace('/', '.');
                        }
                        if (code.endsWith(".java")) {
                            code = code.substring(0, code.length() - 5).replace('/', '.');
                        }
                        try {
                            try {
                                setDownloadState(true);
                                applet = (Applet)loader.loadClass(code).newInstance();
                            } finally {
                                setDownloadState(false);
                            }
                            pdispApplet = getAppletDispatch(applet);
                        } catch (ClassNotFoundException e) {
                            showAppletStatus("load: class " + code + " not found");
                            showAppletException(e);
                            break;
                        } catch (InstantiationException e) {
                            showAppletStatus("load: " + code + " can't be instantiated");
                            showAppletException(e);
                            break;
                        } catch (IllegalAccessException e) {
                            showAppletStatus("load: " + code + " is not public or has no public constructor");
                            showAppletException(e);
                            break;
                        } catch (Exception e) {
                            showAppletStatus("exception: " + e );
                            showAppletException(e);
                            break;
                        } catch (ThreadDeath e) {
                            showAppletStatus("killed");
                            return;
                        } catch (Error e) {
                            showAppletStatus("error: " + e );
                            showAppletException(e);
                            break;
                        }

                        // Stick it in the frame
                        applet.setStub(this);
                        applet.hide();
                        add("Center", applet);
                        status = APPLET_LOAD;
                        showAppletStatus(STATUS_INITED);
                        validate();
                        break;

                      case APPLET_INIT:
                        if (status != APPLET_LOAD) {
                            break;
                        }
                        try {
                            setDownloadState(true);
                            applet.resize(appletSize);
                            applet.init();
                            validate();
                        } finally {
                            setDownloadState(false);
                        }
                        status = APPLET_INIT;
                        showAppletStatus(STATUS_INITED);
                        break;

                      case APPLET_START:
                        if (status != APPLET_INIT) {
                            break;
                        }
                        applet.resize(appletSize);
                        applet.start();
                        validate();
                        applet.show();
                        status = APPLET_START;
                        showAppletStatus(STATUS_STARTED);
                        break;

                      case APPLET_STOP:
                        if (status != APPLET_START) {
                            break;
                        }
                        status = APPLET_INIT;
                        applet.hide();
                        applet.stop();
                        showAppletStatus(STATUS_STOPPED);
                        break;

                      case APPLET_DESTROY:
                        if (status != APPLET_INIT) {
                            break;
                        }
                        status = APPLET_LOAD;
                        applet.destroy();
                        showAppletStatus(STATUS_DESTROYED);
                        break;

                      case APPLET_DISPOSE:
                        if (status != APPLET_LOAD) {
                            break;
                        }
                        status = APPLET_DISPOSE;
                        remove(applet);
                        return;
                    }
                } catch (Exception e) {
                    showAppletStatus("exception: " + e );
                    showAppletException(e);
                } catch (ThreadDeath e) {
                    showAppletStatus("killed");
                    return;
                } catch (Error e) {
                    showAppletStatus("error: " + e );
                    showAppletException(e);
                }
            }
        } finally {
            frames.removeElement(this);
            releaseClassLoader(getCodeBase());
            releaseAppletDispatch(pdispApplet);
            dispose();
            jolHost = null;
        }
    }

    /**
     * Return true when the applet has been started.
     */
    public boolean isActive() {
        return status == APPLET_START;
    }

    /**
     * Status line. Called by the AppletPanel to provide
     * feedback on the Applet's state.
     */
    protected void showAppletStatus(String status) {
        jolHost.showStatus(status);
    }

    protected void showAppletStatus(int id) {
        jolHost.showSystemStatus(id, false, null);
    }

    protected void showLoaderStatus(int id, String arg) {
        jolHost.showSystemStatus(id, true, arg);
    }

    protected void setDownloadState(boolean downloading) {
        jolHost.setDownloadState(downloading);
    }

    /**
     * Called by the AppletPanel to provide
     * feedback when an exception has happend.
     */
    protected void showAppletException(Throwable t) {
        t.printStackTrace();
    }

    /**
     * The class loaders
     */
    private static Vector classloaders = new Vector();

    private static int oldloadersCount = 0;

    private final static int CLASSLOADER_KEEP_ALIVE_COUNT = 3;

    /**
     * Get a class loader.
     */
    AppletBaseClassLoader getClassLoader() throws MalformedURLException
    {
		ClassLoaderEntry ce;
        String cabbase = getParameter("cabbase");
        URL codebase = null;

        // Get an URL to the codebase.
        if (cabbase == null) {
            codebase = getCodeBase();
        } else {
            codebase = new URL(getCodeBase(), cabbase);
        }

        // Synchronize on the classloader object.
        synchronized (classloaders) {
            int i;
            int size = classloaders.size();

            for (i = 0; i < size; i++) {
                ce = (ClassLoaderEntry)classloaders.elementAt(i);

                if (codebase.equals(ce.codebase)) {
                    if (ce.countRefs == 0)
                        oldloadersCount--;

                    ce.countRefs++;
                    return ce.loader;
                }
            }

            // Create a new classloader...
            AppletBaseClassLoader abcl;

            if (cabbase == null) {
                abcl = new AppletClassLoader(codebase);
            } else {
                abcl = new CabClassLoader(codebase);
            }

			try {
				abcl.setSecureState(documentURL.getProtocol(), codebase.getProtocol());
			} catch (Throwable e) {
			}

            ce = new ClassLoaderEntry( codebase, abcl );

            // ...and add it to the list of classloaders.
            classloaders.addElement(ce);
        }

        return ce.loader;
    }

    static synchronized void releaseClassLoader(URL codebase) {

        int i;
        int size = classloaders.size();

        for (i = 0; i < size; i++) {

            ClassLoaderEntry ce = (ClassLoaderEntry)classloaders.elementAt(i);

            if (codebase.equals(ce.codebase)) {

                // Push classloaders with a refcount of zero to the end of the
                // array.
                ce.countRefs--;
                classloaders.removeElementAt(i);
                classloaders.addElement(ce);

                if (ce.countRefs == 0)
                    oldloadersCount++;

                // If we've exceeded the keep alive count, then remove the
                // first element with a reference count of zero.
                while (oldloadersCount >= CLASSLOADER_KEEP_ALIVE_COUNT) {

                    for (i = 0; i < size; i++) {

                        ClassLoaderEntry oldce = (ClassLoaderEntry)
                            classloaders.elementAt(i);

                        if (oldce.countRefs == 0) {
                            classloaders.removeElementAt(i);
                            oldloadersCount--;
                            break;
                        }
                    }
                }

                break;
            }
        }
    }

    /**
     * Gets the document URL.
     */
    public URL getDocumentBase()
    {
        return documentURL;
    }

    /**
     * Gets the base URL.
     */
    public URL getCodeBase()
    {
        return baseURL;
    }

    /**
     * Gets a parameter of the applet.
     */
    public String getParameter(String name)
    {
        return (String)atts.get(name.toLowerCase());
    }

    /**
     * Context for this applet; one is created on demand per frame.
     */
    private BrowserAppletContext appletContext;

    /**
     * Gets a handler to the applet's context.
     */
    public AppletContext getAppletContext()
    {
        if (appletContext == null) {
            appletContext = new BrowserAppletContext(this);
        }
        return appletContext;
    }

    public void appletResize(int width, int height)
    {
    }

    /**
     * Get an applet by name.
     */
    public Applet getApplet(String name)
    {
        AppletSecurity security = (AppletSecurity)System.getSecurityManager();
        name = name.toLowerCase();
        for (Enumeration e = frames.elements() ; e.hasMoreElements() ;) {
            BrowserAppletFrame f = (BrowserAppletFrame)e.nextElement();
            String param = f.getParameter("name");
            if (param != null) {
                param = param.toLowerCase();
            }
            if (name.equals(param) && f.getDocumentBase().equals(getDocumentBase())) {
                try {
                    security.checkConnect(getCodeBase().getHost(), f.getCodeBase().getHost());
                    return f.applet;
                } catch (SecurityException ee) {
                }
            }
        }
        return null;
    }

    /**
     * Return an enumeration of all the accessible
     * applets on this page.
     */
    public Enumeration getApplets()
    {
        AppletSecurity security = (AppletSecurity)System.getSecurityManager();
        Vector v = new Vector();
        for (Enumeration e = frames.elements() ; e.hasMoreElements() ;) {
            BrowserAppletFrame f = (BrowserAppletFrame)e.nextElement();
            if (f.getDocumentBase().equals(getDocumentBase())) {
                try {
                    security.checkConnect(getCodeBase().getHost(), f.getCodeBase().getHost());
                    v.addElement(f.applet);
                } catch (SecurityException ee) {
                }
            }
        }
        return v.elements();
    }

    byte[] getDataFileFromLoader( URL url )
    {
        String stringBase  = baseURL.toString();
        String stringUrl   = url.toString();
        String stringFile  = null;
        
		if ( stringBase.length() == stringUrl.length() ) {
            return null;
        }

        if ( stringUrl.startsWith(stringBase) ) {
            stringFile = stringUrl.substring( stringBase.length() );
        } else {
			return null;
		}

        return loader.getDataItem( stringFile );
    }

    Hashtable audioHash = new Hashtable();

    /**
     * Get a clip from the audio clip cache.
     */
    synchronized AudioClip getAudioClipFromCache(URL url)
    {
		System.getSecurityManager().checkConnect(url.getHost(), url.getPort());
        AudioClip clip = (AudioClip)audioHash.get(url);

        if (clip != null) {
            return clip;
        }

        if (loader.isDataStore()) {
            byte data[] = getDataFileFromLoader(url);
            if (data != null) {
                clip = new AppletAudioClip(url, data);
            }
        }

        if (clip == null) {
            clip = new AppletAudioClip(url);
        }

        audioHash.put(url, clip );

        return clip;
    }

    Hashtable imgHash = new Hashtable();

    /**
     * Get an image from the static image cache.
     */
    synchronized Image getImageFromHash(URL url)
    {
        System.getSecurityManager().checkConnect(url.getHost(), url.getPort());

        Image img = (Image)imgHash.get(url);

        if (img != null) {
            return img;
        }

        ImageProducer imgProducer = null;

        try {
            try {
                if (loader.isDataStore()) {
                    imgProducer = new ByteArrayImageSource(url.toString(), getDataFileFromLoader(url));
                }
            } catch (Exception e) {
            }

            if (imgProducer == null) {
                URLConnection conn = url.openConnection();
                conn.setUseCaches(true);
                imgProducer = new URLImageSource(url, conn);
            }

            if (imgProducer != null) {
                img = Toolkit.getDefaultToolkit().createImage(imgProducer);
                imgHash.put(url, img);
            }
        } catch (Exception e) {
        }

        return img;
    }

    static boolean isFirstTimeInited = false;

    static synchronized void firstTimeInit()
    {
        if (isFirstTimeInited)
            return;

        isFirstTimeInited = true;

        //  Install our WININET stream handler factory.  If this fails, the
        //  system will fall back on the existing factory or use the default
        //  Sun handlers.
        try {
            URL.setURLStreamHandlerFactory(new WininetStreamHandlerFactory());
        } catch (Exception e) {
        }

        //  Redirect stdout and stderr to a file called javalog.txt in the
        //  user's Java directory.  PrintStream's print methods are
        //  synchronized so two threads writing to stdout and stderr will be
        //  okay.
        if ("true".equalsIgnoreCase(System.getProperty("com.ms.applet.enable.logging"))) {
            try {
                String logdir = System.getProperty("java.home");
                PrintStream ps = new PrintStream(new BufferedOutputStream(new
                    FileOutputStream(new File(logdir, "javalog.txt"))), true);
                System.out = ps;
                System.err = ps;
            } catch (Exception e) {
            }
        }

        Properties props = new Properties(System.getProperties());

        //  Define which packages can be extended by applets.
        props.put("package.restrict.definition.java", "true");
        props.put("package.restrict.definition.sun", "true");
        props.put("package.restrict.definition.com.ms", "true");

        //  Define properties that Java applets can access.  Currently, this
        //  is the same set as available from Sun's AppletViewer.
        props.put("java.version.applet", "true");
        props.put("java.vendor.applet", "true");
        props.put("java.vendor.url.applet", "true");
        props.put("java.class.version.applet", "true");
        props.put("os.name.applet", "true");
        props.put("os.version.applet", "true");
        props.put("os.arch.applet", "true");
        props.put("file.separator.applet", "true");
        props.put("path.separator.applet", "true");
        props.put("line.separator.applet", "true");
        props.put("browser.applet", "true");

        System.setProperties(props);

        //  Because we don't use the Sun HTTP protocol handlers to download
        //  data, we may not touch the java.net.InetAddress class and thus not
        //  call its class initializer until some later point.  At that time,
        //  a SecurityManager has been installed, so its class initializer is
        //  not able to complete normally-- specifically, it cannot resolve
        //  the local host address.  Thus, we touch it once here so that the
        //  class may initialize BEFORE the SecurityManager is installed.
        try {
            InetAddress.getLocalHost();
        } catch (Exception e) {
        }

        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new AppletSecurity());
        }
    }

    public IJavaObjectLoaderHost getLoaderHost()
    {
        return jolHost;
    }

    IJavaObjectLoaderHost jolHost;
}
