//
//  FontX.java
//
//  Copyright (c) 1996, Microsoft Corporation
//
// notes:
// 1.	cant override getFont.
// 2.	going to have to eventually override ALL methods of java.awt.Font,
//		leave place holders in code now.
//
package com.ms.awt;
import java.awt.Font;

public class FontX extends Font
{
    // public static final int PLAIN	= 0;
    // public static final int BOLD	= 1;
	// public static final int ITALIC	= 2;
    // private int pData;
    // private String family;
    // protected String name;
    // protected int style;
    // protected int size;
	public static final int EMBEDDED = 1;
	public static final int STRIKEOUT = 2;
	public static final int UNDERLINE = 4;
	private static final int FONTXFONT = 0x8000;
	
    private native static int pNewFontEnumeration();
	private native static String pGetFontEnumeratedFamily(int i);
	private native boolean pIsTypeable(int lang);

	private int flags = FONTXFONT;		// FONTXFONT is used by the native side.
	
	//========================================================================
	
	public FontX(String name, int style, int size) 
    {
    	super(name, style, size);
    }

    public FontX(String name, int style, int size, boolean bEmbed) 
    {
    	super(name, style, size);
    	if ( bEmbed )
    		flags |= 1;
    	else
    		flags &= 0xfffe;
    }

    public FontX(String name, int style, int size, int theNewFlags) 
    {
    	super(name, style, size);
		flags |= theNewFlags;
    }

	//========================================================================

	public boolean isTypeable(int language)
	{
		return pIsTypeable(language);
	}

	//========================================================================
	public int getFlags()
	{
		return flags;
	}
	
	//========================================================================
/*
 * All the following need modifying for the new class - for tt extensions
 *
    public String getFamily() 
    {
		return super.getFamily();
    }

	//========================================================================
    public String getName() 
    {
		return super.getName();
    }

	//========================================================================
    public int getStyle() 
    {
		return super.getStyle();
    }

	//========================================================================
    public int getSize() 
    {
		return super.getSize();
    }

	//========================================================================
    public boolean isPlain() 
    {
	return 	super.isPlain();
    }

	//========================================================================
    public boolean isBold() 
    {
		return super.isBold();
    }

	//========================================================================
    public boolean isItalic() 
    {
		return super.isItalic();
    }

	//========================================================================
	public int hashCode() 
    {
		return (super.getName()).hashCode() ^ style ^ size;
	}
*/
	//========================================================================
	
    public boolean equals(Object obj) 
    {
    	if( obj instanceof FontX )
    		return super.equals(obj);
    	else if (obj instanceof Font) 
    		return super.equals(obj);
    	else
			return false;
    }

	//========================================================================
	
    public String toString() 
    {
		String	strStyle;

		if (isBold()) 
			strStyle = isItalic() ? "bolditalic" : "bold";
			
		else 
		    strStyle = isItalic() ? "italic" : "plain";

		return getClass().getName() + 
					"[family=" + getFamily() + 
					",name=" + getName() + 
					",style=" + strStyle + 
					",size=" + getSize() + 
					(((flags & EMBEDDED)!=0) ?", Embedded" :"") +
					(((flags & STRIKEOUT)!=0) ?", Strikeout" :"") +
					(((flags & UNDERLINE)!=0) ?", Underline" :"") +
					"]";
    }
    
	//========================================================================
	
    public static String[] getFontList() 
    {
    	int listSize = pNewFontEnumeration();
    	
    	String list[] = new String[listSize + 6];
    	list[0] = "Dialog";
    	list[1] = "Helvetica";
    	list[2] = "TimesRoman";
		list[3] = "Courier";
    	list[4] = "DialogInput";
    	list[5] = "ZapfDingbats";

    	for(int i=0; i<listSize; i++)
    		list[6+i] = pGetFontEnumeratedFamily(i);
		return list;
    }
	
	//========================================================================
}

