//
//  SystemX.java
//
//  Copyright (c) 1996, Microsoft Corporation
//
package com.ms.lang;

import java.applet.*;
import java.awt.peer.ComponentPeer;
import com.ms.awt.peer.ComponentPeerX;

/**
 * This Class provides system enhancements for extended functionality 
 * Currently the methods in the class are all keyboard/Unicde related.
 * This Class cannot be instantiated or subclassed because all of the methods
 * and variables are static.
 * @version 	1.00, 7/25/96
 * @author	David Silver
 */
public final class SystemX
{
    /**
     * The Afrikaans language constant.
     */
	public static final int LANG_AFRIKAANS                  = 0x0436 ;
    /**
     * The Albanian language constant.
     */
	public static final int LANG_ALBANIAN                   = 0x041c ;
    /**
     * The Basque language constant.
     */
	public static final int LANG_BASQUE                     = 0x042d ;
    /**
     * The Bulgarian language constant.
     */
	public static final int LANG_BULGARIAN                  = 0x0402 ;
    /**
     * The Byelorussian language constant.
     */
	public static final int LANG_BYELORUSSIAN               = 0x0423 ;
    /**
     * The Catalan language constant.
     */
	public static final int LANG_CATALAN                    = 0x0403 ;
    /**
     * The generic Chinese language constant.
     */
	public static final int LANG_CHINESE                    = 0x0404 ;
    /**
     * The Traditional Chinese language constant.
     */
	public static final int SUBLANG_CHINESE_TRADITIONAL     = 0x0404 ;    /* Chinese (Taiwan) */
	public static final int SUBLANG_CHINESE_SIMPLIFIED      = 0x0804 ;    /* Chinese (PR China) */
	public static final int SUBLANG_CHINESE_HONGKONG        = 0x0C04 ;    /* Chinese (Hong Kong) */
	public static final int SUBLANG_CHINESE_SINGAPORE       = 0x1004 ;    /* Chinese (Singapore) */
	public static final int LANG_CROATIAN                   = 0x041a ;
	public static final int LANG_CZECH                      = 0x0405 ;
	public static final int LANG_DANISH                     = 0x0406 ;
	public static final int LANG_DUTCH                      = 0x0413 ;
	public static final int SUBLANG_DUTCH_BELGIAN           = 0x0813 ;	 /* Dutch (Belgian) */
	public static final int LANG_ENGLISH                    = 0x0409 ;
	public static final int SUBLANG_ENGLISH_US              = 0x0409 ;    /* English (USA) */
	public static final int SUBLANG_ENGLISH_UK              = 0x0809 ;    /* English (UK) */
	public static final int SUBLANG_ENGLISH_AUS             = 0x0c09 ;    /* English (Australian) */
	public static final int SUBLANG_ENGLISH_CAN             = 0x1009 ;    /* English (Canadian) */
	public static final int SUBLANG_ENGLISH_NZ              = 0x1409 ;    /* English (New Zealand) */
	public static final int SUBLANG_ENGLISH_EIRE            = 0x1809 ;    /* English (Irish) */
	public static final int LANG_ESTONIAN                   = 0x0425 ;
	public static final int LANG_FINNISH                    = 0x040b ;
	public static final int LANG_FRENCH                     = 0x040c ;
	public static final int SUBLANG_FRENCH                  = 0x040c ;	/* French */
	public static final int SUBLANG_FRENCH_BELGIAN          = 0x080c ;  /* French (Belgian) */
	public static final int SUBLANG_FRENCH_CANADIAN         = 0x0c0c ;  /* French (Canadian) */
	public static final int SUBLANG_FRENCH_SWISS            = 0x100c ;  /* French (Swiss) */
	public static final int LANG_GERMAN                     = 0x0407 ;
	public static final int SUBLANG_GERMAN                  = 0x0407 ;   /* German */
	public static final int SUBLANG_GERMAN_SWISS            = 0x0807 ;   /* German (Swiss) */
	public static final int SUBLANG_GERMAN_AUSTRIAN         = 0x0c07 ;   /* German (Austrian) */
	public static final int LANG_GREEK                      = 0x0408 ;
	public static final int LANG_HEBREW						= 0x040D ;
	public static final int LANG_HUNGARIAN                  = 0x040e ;
	public static final int LANG_ICELANDIC                  = 0x040f ;
	public static final int LANG_INDONESIAN                 = 0x0421 ;
	public static final int LANG_ITALIAN                    = 0x0410 ;
	public static final int SUBLANG_ITALIAN                 = 0x0410 ;    /* Italian */
	public static final int SUBLANG_ITALIAN_SWISS           = 0x0810 ;    /* Italian (Swiss) */
	public static final int LANG_JAPANESE                   = 0x0411 ;
	public static final int LANG_KOREAN                     = 0x0412 ;
	public static final int LANG_LATVIAN                    = 0x0426 ;
	public static final int LANG_LITHUANIAN                 = 0x0427 ;
	public static final int LANG_NORWEGIAN                  = 0x0414 ;
	public static final int SUBLANG_NORWEGIAN_BOKMAL        = 0x0414 ;    /* Norwegian (Bokmal) */
	public static final int SUBLANG_NORWEGIAN_NYNORSK       = 0x0814 ;    /* Norwegian (Nynorsk) */
	public static final int LANG_POLISH                     = 0x0415 ;
	public static final int LANG_PORTUGUESE                 = 0x0416 ;
	public static final int SUBLANG_PORTUGUESE              = 0x0416 ;    /* Portuguese */
	public static final int SUBLANG_PORTUGUESE_BRAZILIAN    = 0x0816 ;    /* Portuguese (Brazilian) */
	public static final int LANG_ROMANIAN                   = 0x0418 ;
	public static final int LANG_RUSSIAN                    = 0x0419 ;
	public static final int LANG_SLOVAK                     = 0x041b ;
	public static final int LANG_SLOVENIAN                  = 0x0424 ;
	public static final int LANG_SORBIAN                    = 0x042e ;
	public static final int LANG_SPANISH                    = 0x040a ;
	public static final int SUBLANG_SPANISH                 = 0x040a ;    /* Spanish (Castilian) */
	public static final int SUBLANG_SPANISH_MEXICAN         = 0x080a ;    /* Spanish (Mexican) */
	public static final int SUBLANG_SPANISH_MODERN          = 0x0c0a ;    /* Spanish (Modern) */
	public static final int LANG_SWEDISH                    = 0x041d ;
	public static final int LANG_THAI                       = 0x041e ;
	public static final int LANG_TURKISH                    = 0x041f ;
	public static final int LANG_UKRAINIAN                  = 0x0422 ;
	public static final int LANG_ARABIC						= 0x0401 ;

    private static char baseCodePage[];
    private static boolean isDBCSBase ;
    
    private native static boolean pSetKeyboardLayout( int hwnd, int idLanguage );
    private native static int pGetNumKeyboardLayouts();
    private native static int pGetKeyboardLayouts(int array[]);
    private native static int pGetKeyboardLayout();
	private native static int pAnsiToUnicode( byte[] B, char[] C, int off, int len, int cp );
	private native static boolean isBaseDBCS();
	private native static int pUnicodeToAnsi( char[] C, byte[] B, int off, int len, int cp );
	private native static int pQueryUnicodeToAnsi( char[] C, int off, int len, int cp );
	public native static boolean isLocalCharDBCSLeadByte(byte B);
    
	//========================================================================
	// dont let anyone instantiate this!
    private SystemX() 
    {
    	isDBCSBase = isBaseDBCS();
    	if( isDBCSBase )
    		baseCodePage = null;
		else
		{
    		baseCodePage = new char[256];
    		byte b[] = new byte[256];
    		
    		for( int i=0; i<256; i++)
    			b[i] = (byte)i;

    		if( pAnsiToUnicode(b, baseCodePage, 0, 256, 0) != 256 )
    			baseCodePage = null;
    		}
    }

	//========================================================================

    /**
     * Converts a string from a local codepage to Unicode.
     * Java internally uses the Unicode encoding system.  Unfortunately this
     * means more than just using 16-bits per character in the local code page.
     * For example, character (149) in the Windows codepage used in America
     * (codepage 1252) is a bullet character. This is not true on the Macintosh,
     * nor on many Unix systems, so if you try to hardcode this value, the bullet
     * will not appear correctly on other platforms. 
     *
     * @param local the local codepage string
     * @return 	the Unicode representation of the string.
     */
	public static String LocalStringToJavaString( String local )
	{
		int len = local.length();
		char C[] = new char[len];
		byte B[] = new byte[len];
		
		local.getBytes(0, len, B, 0);
		
		if( baseCodePage != null )
		{
			for(int i=0; i<len; i++)
			C[i] = baseCodePage[B[i]];
		}
		else
		{
			pAnsiToUnicode(B, C, 0, len, 0);
		}
		return new String(C, 0, len);
	}
	
	//========================================================================

    /**
     * Converts a byte array from a local codepage to a Unicode filled char array
     * Java internally uses the Unicode encoding system.  Unfortunately this
     * means more than just using 16-bits per character in the local code page.
     * See LocalStringToJavaString()
     *
     * @param B 	   the local codepage string
     * @return 	char[] the Unicode representation of the array.
     */
	public static char[] LocalStringToJavaString( byte B[] )
	{
		return LocalStringToJavaString(B, 0, B.length);
	}
	
	public static char[] LocalStringToJavaString( char B[] )
	{
		return LocalStringToJavaString(B, 0, B.length);
	}
	
    /**
     * Converts a part of a byte array from a local codepage to a Unicode 
     * filled char array. Java internally uses the Unicode encoding system.  
     * Unfortunately this means more than just using 16-bits per character 
     * in the local code page.
     * See LocalStringToJavaString()
     *
     * @param B 		the local codepage string
     * @param off 		starting point within the array to 
     * @param len		length of the segment to convert.
     * @return char[] 	the Unicode representation of the array.
     */
	public static char[] LocalStringToJavaString( byte B[], int off, int len )
	{
		char C[] = new char[len];
		
		if( baseCodePage != null )
		{
			for(int i=0; i<len; i++)
			C[i] = baseCodePage[B[i]+off];
		}
		else
		{
			pAnsiToUnicode(B, C, off, len, 0);
		}
		return C;
	}
	
	public static char[] LocalStringToJavaString( char B[], int off, int len )
	{
		char C[] = new char[len];
		
		if( baseCodePage != null )
		{
			for(int i=0; i<len; i++)
			C[i] = baseCodePage[B[i]+off];
		}
		else
		{
			// strip high bytes
			byte B1[] = new byte[len];
			for(int i=0; i<len; i++)
				B1[i] = (byte)B[i];
				
			pAnsiToUnicode(B1, C, off, len, 0);
		}
		return C;
	}
	
    /**
     * Converts a byte from a local codepage to a Unicode char.Java internally 
     * uses the Unicode encoding system.  Unfortunately this means more than 
     * just using 16-bits per character in the local code page. 
     * See LocalStringToJavaString()
     *
     * @param local the local codepage string
     * @return 	the Unicode representation of the array.
     */
	public static char LocalStringToJavaString( byte local )
	{
		if( baseCodePage != null )
			return baseCodePage[local];
		else
		{
			byte B[] = new byte[1];
			char C[] = new char[1];

			B[0] = local;
			pAnsiToUnicode(B, C, 0, 1, 0);
			return C[0];
		}
	}
	
	//========================================================================

    /**
     * Converts a Java character to a character in the local codepage 
     *
     * @param local	the Java character
     * @return the local codepage string
     */
	public static byte JavaStringToLocalString( char Uni )
	{
		byte B[] = new byte[1];
		char C[] = new char[1];

		C[0] = Uni;
		pUnicodeToAnsi(C, B, 0, 1, 0);
		return B[0];
	}
	
    /**
     * Converts a char array to a byte array in the local codepage
     *
     * @param B 	   the Java characters string
     * @return 	char[] the local representation of the array.
     */
	public static byte[] JavaStringToLocalString( char C[] )
	{
		return JavaStringToLocalString(C, 0, C.length);
	}
	
    /**
     * Converts a char array to a byte array in the local codepage
     * See LocalStringToJavaString()
     *
     * @param B 	   the Java characters string
     * @param off 		starting point within the array to 
     * @param len		length of the segment to convert.
     * @return 	char[] the local representation of the array.
     */
	public static byte[] JavaStringToLocalString( char C[], int off, int len )
	{
		byte B[] = new byte[pQueryUnicodeToAnsi(C, off, len, 0)];
		
		pUnicodeToAnsi(C, B, off, len, 0);
		return B;
	}
	
	//========================================================================

    /**
     * Find the number of active keyboard languages in the system.
     */
    public static int getNumKeyboardLanguages() 
    {
		return pGetNumKeyboardLayouts();
    }

	//========================================================================

    /**
     * Return a list of the keyboard Languages currently active in the system.
	 *
     * @param array[]	a list of keyboard language ID's
     * @return 			the number of active languages..
     */
    public static int getKeyboardLanguages(int array[]) 
    {
		return pGetKeyboardLayouts(array);
    }
	//========================================================================
	
    /**
     * Gets the currently active keyboard Language.
     */
    public static int getKeyboardLanguage() 
    {
		return pGetKeyboardLayout();
    }
    
	//========================================================================
	
    /**
     * Set a new active keyboard language.
     *
     * @param App			the Applet to set the Language for.
     * @param udLanguage	the Langauge to activate.
     * @return	If the action was successful
     */
    public static boolean setKeyboardLanguage( Applet App, int idLanguage ) 
    {
		ComponentPeer p = App.getPeer();
		if( p != null )
		{
			ComponentPeerX peer = (ComponentPeerX)p;
			int hwnd = peer.gethwnd();
    	
			return pSetKeyboardLayout(hwnd, idLanguage);
		}
		else
			return false;
    }
    
	//========================================================================
}
