//
//  HttpInputStream.java
//
//  Copyright (c) 1996, Microsoft Corporation
//

package com.ms.net.wininet.http;

import java.io.*;
import java.net.URL;
import com.ms.net.wininet.URLUtils;

class HttpInputStream extends InputStream
{
    //  WININET handles used by the native code.
    private int connectHandle = 0;
    private int requestHandle = 0;

    //  Buffer used when reading byte at a time.
    private byte temp[] = new byte[1]; 

    HttpInputStream(HttpURLConnection conn) throws IOException
    {
        URL url = conn.getURL();
        byte postBuffer[];

        //  Encode the URL with Wininet's canonicalize API.
        String s = url.toString();
        s = URLUtils.canonicalizeURL(s, URLUtils.ICU_ENCODE_SPACES_ONLY);
        if (s != null) {
            url = new URL(s);
        }

        if (conn.outputStream != null) {
            postBuffer = conn.outputStream.toByteArray();
        } else {
            postBuffer = null;
        }

        connect(url.getHost(), url.getPort(), url.getFile(), conn.secure,
            conn.getUseCaches(), conn.getAllowUserInteraction(), postBuffer);
    }                                            

    private native void connect(String serverName, int serverPort, String
        fileName, boolean secure, boolean useCaches, boolean
        allowUserInteraction, byte postBuffer[]) throws IOException;

    //  This is the same thing java.net.SocketInputStream does when reading
    //  a byte at a time.
    public int read() throws IOException
    {
        int n = read(temp, 0, 1);
        if (n <= 0) {
            return -1;
        }
        return temp[0] & 0xFF;
    }

    public native int read(byte buf[], int off, int len) throws IOException;
    public native long skip(long n) throws IOException;
    public native void close() throws IOException;
    protected native byte[] getResponseHeaders();

    protected void finalize() throws IOException
    {
        if (connectHandle != 0 || requestHandle != 0)
            close();
    }
}
