//
//  HttpURLConnection.java
//
//  Copyright (c) 1996, Microsoft Corporation
//

package com.ms.net.wininet.http;

import java.io.*;
import java.net.URL;
import sun.net.www.MessageHeader;

class HttpURLConnection extends java.net.URLConnection
{
    //  TRUE if https: connection.
    protected boolean secure = false;

    HttpURLConnection(URL u, boolean secure)
    {
        super(u);
        this.secure = secure;
    }

    //  When using WININET, there's not a state where you're connected to the
    //  server but not in the middle of an HTTP operation such as GET or PUT.
    //  Thus, we currently do no useful work during this phase; everything
    //  interesting is done when the client asks for the InputStream or
    //  OutputStream.
    public void connect() throws IOException
    {
        if (!connected) {
            SecurityManager security = System.getSecurityManager();
            if (security != null) {
                security.checkConnect(url.getHost(), url.getPort());
            }
            //  BUGBUG:  WININET only supports URLs up to 2048 characters, so
            //  limit URL lengths to the below limit.  Future versions should
            //  remove this restriction.
            if (url.toString().length() > 2000) {
                throw new IOException("URL too long");
            }
            connected = true;
        }
    }

    boolean attemptedGetInput = false;
    HttpInputStream inputStream = null;

    public InputStream getInputStream() throws IOException
    {
        //  We only want to try to connect to the server once.  Subsequent
        //  attempts to connect will either use the existing stream object or
        //  throw an exception.
        if (!attemptedGetInput) {
            connect();
            attemptedGetInput = true;
            inputStream = new HttpInputStream(this);
            headers = new MessageHeader(new
                ByteArrayInputStream(inputStream.getResponseHeaders()));
        }
        if (inputStream == null) {
            throw new IOException();
        }
        return inputStream;
    }

    ByteArrayOutputStream outputStream = null;

    public OutputStream getOutputStream() throws IOException
    {
        connect();

        //  If there's already an output stream open, throw an exception.
        if (inputStream != null) {
            throw new IOException("Cannot write output after reading input.");
        }
        //  Only hand out a single output stream for this connection.
        if (outputStream != null) {
            throw new IOException("Cannot open output twice.");
        }

        outputStream = new HttpPostBufferStream(this);
        return (OutputStream) outputStream;
    }

    MessageHeader headers;

    public String getHeaderField(String name)
    {
        String value = null;
        try {
            getInputStream();
            value = headers.findValue(name);
        } catch (Exception e) {
        }
        return value;
    }

    public String getHeaderFieldKey(int n)
    {
        String value = null;
        try {
            getInputStream();
            value = headers.getKey(n);
        } catch (Exception e) {
        }
        return value;
    }

    public String getHeaderField(int n)
    {
        String value = null;
        try {
            getInputStream();
            value = headers.getValue(n);
        } catch (Exception e) {
        }
        return value;
    }
}
